@@ -118,7 +118,9 @@ class Network(object):
     '''
     self.dat['mat'][np.isnan(self.dat['mat'])] = 0
 
-  def load_df(self, df_ini, meta_col=None, meta_row=None):
+  def load_df(self, df_ini, meta_col=None, meta_row=None, col_cats=None,
+              row_cats=None, is_downsampled=False, meta_ds_row=None,
+              meta_ds_col=None):
     '''
     Load Pandas DataFrame.
     '''
@@ -127,15 +129,38 @@ class Network(object):
     # load dataframe
     df = deepcopy(df_ini)
 
-    self.meta_cat = False
+    if hasattr(self, 'meta_col') == False or hasattr(self, 'meta_row') == False:
+      self.meta_cat = False
+
+    if is_downsampled:
+      if meta_ds_col is not None:
+        self.meta_ds_col = meta_ds_col
+      if meta_ds_row is not None:
+        self.meta_ds_row = meta_ds_row
+
+    # define downsampled status
+    self.is_downsampled = is_downsampled
+    # print('load_df: is_downsampled', is_downsampled)
 
     # load metadata
     if isinstance(meta_col, pd.DataFrame):
       self.meta_col = meta_col
+
+      if col_cats is None:
+        self.col_cats = meta_col.columns.tolist()
+      else:
+        self.col_cats = col_cats
+
       self.meta_cat = True
 
     if isinstance(meta_row, pd.DataFrame):
       self.meta_row = meta_row
+
+      if row_cats is None:
+        self.row_cats = meta_row.columns.tolist()
+      else:
+        self.row_cats = row_cats
+
       self.meta_cat = True
 
     data_formats.df_to_dat(self, df, define_cat_colors=True)
@@ -145,6 +170,12 @@ class Network(object):
     Export Pandas DataFrame/
     '''
     df = data_formats.dat_to_df(self)
+
+    # drop tuple categories if downsampling
+    if self.is_downsampled:
+      df.columns = self.dat['nodes']['col']
+      df.index = self.dat['nodes']['row']
+
     return df
 
   def df_to_dat(self, df, define_cat_colors=False):
@@ -348,12 +379,13 @@ class Network(object):
     '''
     normalize_fun.run_norm(self, df, norm_type, axis)
 
-  def downsample(self, df=None, ds_type='kmeans', axis='row', num_samples=100, random_state=1000):
+  def downsample(self, df=None, ds_type='kmeans', axis='row', num_samples=100,
+                 random_state=1000, ds_name='Downsample',
+                 ds_cluster_name='cluster'):
     '''
     Downsample the matrix rows or columns (currently supporting kmeans only). Users can optionally pass in a DataFrame to be downsampled (and this will be incorporated into the network object).
     '''
-
-    return downsample_fun.main(self, df, ds_type, axis, num_samples, random_state)
+    return downsample_fun.main(self, df, ds_type, axis, num_samples, random_state, ds_name, ds_cluster_name)
 
   def random_sample(self, num_samples, df=None, replace=False, weights=None, random_state=100, axis='row'):
     '''
@@ -1101,7 +1133,7 @@ class Network(object):
           inst_color = cat_colors[inst_ct]
           self.set_cat_color(axis=axis, cat_index=cat_index, cat_name=cat_name, inst_color=inst_color)
 
-  def manual_category(self, col=None, row=None):
+  def set_manual_category(self, col=None, row=None):
     '''
     This method is used to tell Clustergrammer2 that the user wants to define
     a manual category interactively using the dendrogram.
@@ -1112,43 +1144,81 @@ class Network(object):
     self.dat['manual_category']['col'] = col
     self.dat['manual_category']['row'] = row
 
-  def get_manual_category(self, axis, cat_title):
-    try:
-      export_dict = {}
+  def ds_to_original_meta(self, axis):
+    clusters = self.meta_ds_col.index.tolist()
+
+    man_cat_title = self.dat['manual_category'][axis]
+
+    for inst_cluster in clusters:
+
+        # get the manual category from the downsampled data
+        inst_man_cat = self.meta_ds_col.loc[inst_cluster, man_cat_title]
+
+        # find original labels that are assigned to this cluster
+        found_labels = self.meta_col[self.meta_col[self.ds_name] == inst_cluster].index.tolist()
+
+        # update manual category
+        self.meta_col.loc[found_labels, man_cat_title] = inst_man_cat
+
+  def get_manual_category(self):
+
+    for axis in ['col']:
 
-      # Category Names
-      #######################
       try:
-        export_dict[cat_title] = pd.Series(json.loads(self.widget_instance.custom_cat)[axis][cat_title])
+      ###########################################################
 
-        if hasattr(self, 'meta_cat') == True:
+        export_dict = {}
 
-          if axis == 'row':
-            print('saving to self')
-            self.meta_row[cat_title] = export_dict[cat_title]
+        inst_nodes = self.dat['nodes'][axis]
 
-          elif axis == 'col':
-            print('saving to self')
-            self.meta_col[cat_title] = export_dict[cat_title]
+        cat_title = self.dat['manual_category'][axis]
+
+        # Category Names
+        try:
+        ###########################################################
+
+
+          export_dict[cat_title] = pd.Series(json.loads(self.widget_instance.custom_cat)[axis][cat_title])
+
+
+          if hasattr(self, 'meta_cat') == True:
+
+            if axis == 'row':
+              if self.is_downsampled == False:
+                self.meta_row.loc[inst_nodes, cat_title] = export_dict[cat_title]
+              else:
+                self.meta_ds_row.loc[inst_nodes, cat_title] = export_dict[cat_title]
+                self.ds_to_original_meta(axis)
+
+            elif axis == 'col':
+              if self.is_downsampled == False:
+                self.meta_col.loc[inst_nodes, cat_title] = export_dict[cat_title]
+              else:
+                self.meta_ds_col.loc[inst_nodes, cat_title] = export_dict[cat_title]
+                self.ds_to_original_meta(axis)
+
+
+        except:
+          pass
+          # print('unable to load', axis ,' category, please check title')
+
+        # Category Colors
+        #######################
+        ini_new_colors = json.loads(self.widget_instance.custom_cat)[axis + '_cat_colors']
+        # drop title from category colors
+        export_dict['cat_colors'] = {}
+        for inst_cat in ini_new_colors:
+          if (': ' in inst_cat):
+            export_dict['cat_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
+          else:
+            export_dict['cat_colors'][inst_cat] = ini_new_colors[inst_cat]
+
+        if hasattr(self, 'meta_cat') == False:
+          return export_dict
 
       except:
-        print('unable to load category, please check title')
-
-      # Category Colors
-      #######################
-      ini_new_colors = json.loads(self.widget_instance.custom_cat)[axis + '_cat_colors']
-      # drop title from category colors
-      export_dict['cat_colors'] = {}
-      for inst_cat in ini_new_colors:
-        if (': ' in inst_cat):
-          export_dict['cat_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
-        else:
-          export_dict['cat_colors'][inst_cat] = ini_new_colors[inst_cat]
-
-      if hasattr(self, 'meta_cat') == False:
-        return export_dict
-    except:
-        print('please set custom category')
+          # print('please set custom category')
+          pass
 
 
   @staticmethod
