@@ -14,9 +14,18 @@ def df_to_dat(net, df, define_cat_colors=False):
   net.dat['nodes']['row'] = df.index.tolist()
   net.dat['nodes']['col'] = df.columns.tolist()
 
-  # parse categories from tuple
-  ##################################
+  # print('checking ds status')
+  # print('is_downsampled', net.is_downsampled)
+  # print('meta_cat', net.meta_cat)
+  # print(hasattr(net, 'meta_ds_col'))
+  # print(hasattr(net, 'meta_ds_row'))
+
+  # if net.meta_cat == False or net.is_downsampled:
   if net.meta_cat == False:
+
+    # tuple cats
+    ##################################
+
     for axis in ['row', 'col']:
 
       inst_nodes = net.dat['nodes'][axis]
@@ -30,8 +39,8 @@ def df_to_dat(net, df, define_cat_colors=False):
 
         # get the number of categories from the length of the tuple
         # subtract 1 because the name is the first element of the tuple
-        num_cat = len(inst_nodes[0]) - 1
-        for inst_cat in range(num_cat):
+        num_cats = len(inst_nodes[0]) - 1
+        for inst_cat in range(num_cats):
           cat_name = 'cat-' + str(inst_cat)
           cat_index = inst_cat + 1
           cat_values = [x[cat_index] for x in inst_nodes]
@@ -41,7 +50,9 @@ def df_to_dat(net, df, define_cat_colors=False):
         net.dat['nodes'][axis] = [x[0] for x in inst_nodes]
 
   else:
-    # print('meta_cat!!!!!!!!')
+
+    # meta_cats
+    ##########################
 
     for axis in ['row', 'col']:
 
@@ -53,21 +64,48 @@ def df_to_dat(net, df, define_cat_colors=False):
         net.dat['node_info'][axis]['full_names'] = df.columns.tolist()
 
       if axis == 'row':
-        inst_cats = net.meta_row.columns.tolist()
+        # inst_cats = net.meta_row.columns.tolist()
+        inst_cats = net.row_cats
       else:
-        inst_cats = net.meta_col.columns.tolist()
+        # inst_cats = net.meta_col.columns.tolist()
+        inst_cats = net.col_cats
 
-      num_cat = len(inst_cats)
-      for inst_cat in range(num_cat):
+      num_cats = len(inst_cats)
+
+      # if axis == 'row':
+      #   num_cats = len(net.row_cats)
+      # elif axis == 'col':
+      #   num_cats = len(net.col_cats)
+
+      for inst_cat in range(num_cats):
         cat_name = 'cat-' + str(inst_cat)
         cat_index = inst_cat + 1
-        # cat_value = [x[cat_index] for x in inst_nodes]
 
         cat_title = inst_cats[inst_cat]
         if axis == 'row':
-          cat_values = net.meta_row[cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+          # cat_values = net.meta_row.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+          if net.is_downsampled:
+            if hasattr(net, 'meta_ds_row'):
+              cat_values = net.meta_ds_row.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+            else:
+              cat_values = net.meta_row.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+
+          # detault with no downsampling
+          else:
+            cat_values = net.meta_row.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
         else:
-          cat_values = net.meta_col[cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+          # cat_values = net.meta_col.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+          if net.is_downsampled:
+            if hasattr(net, 'meta_ds_col'):
+              # print(inst_nodes)
+
+              cat_values = net.meta_ds_col.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+            else:
+              cat_values = net.meta_col.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
+
+          # detault with no downsampling
+          else:
+            cat_values = net.meta_col.loc[inst_nodes, cat_title].apply(lambda x: cat_title + ': ' + x).values.tolist()
 
         net.dat['node_info'][axis][cat_name] = cat_values
 
